
/*################################################################################################
 *##################                                                            ##################
 *##################                     Genetic engine                         ##################
 *##################                                                            ##################
 *##################                     Implementation                         ##################
 *##################                                                            ##################
 *################################################################################################
 */

/*
 * RIGEL: Representation-Independant Genetic Engine Library
 *
 * Copyright (C) 2005-2005 Yann LANDRIN-SCHWEITZER a.k.a. Varkhan
 *
 *
 * This library is free software;  you can redistribute it and/or modify it
 * under  the terms of the  GNU Lesser General Public License  as published
 * by the  Free Software Foundation;  either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This  library  is  distributed  in  the hope that it will be useful, but
 * WITHOUT   ANY   WARRANTY;   without   even   the   implied  warranty  of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library;  if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 *
 * Author:   Yann LANDRIN-SCHWEITZER
 * Contact:  varkhan@free.fr
 * Homepage: http://www.varkhan.net/
 */


#include <errno.h>
#include <stdlib.h>
#include <math.h>

#include <RIGEL/Genetics.h>

#include "Config.h"

/* IEEE Not A Number.  */

#if __GNUC_PREREQ(3,3)
# define NAN	(__builtin_nanf(""))
#elif defined __GNUC__
# define NAN \
  (__extension__                                                            \
   ((union { unsigned __l __attribute__((__mode__(__SI__))); float __d; })  \
    { __l: 0x7fc00000UL }).__d)
#else
# include <endian.h>
# if __BYTE_ORDER == __BIG_ENDIAN
#  define __nan_bytes		{ 0x7f, 0xc0, 0, 0 }
# endif
# if __BYTE_ORDER == __LITTLE_ENDIAN
#  define __nan_bytes		{ 0, 0, 0xc0, 0x7f }
# endif
static union { unsigned char __c[4]; float __d; } __nan_union
    __attribute_used__ = { __nan_bytes };
# define NAN	(__nan_union.__d)
#endif	/* GCC.  */


//################################################################################
//###
//###  Support functions
//###


RIGEL_Population _RIGEL_PopulationAlloc(unsigned long size, unsigned long hist) {
	RIGEL_Population pop ;
	pop = malloc(sizeof(struct S_RIGEL_Population)) ;
	if(pop==NULL) {
		return NULL ;
	}
	pop->iface = RIGEL_InitInterface() ;
	pop->param = NULL ;
	pop->date = 0 ;
	pop->nwid = 0 ;
	if(size==0) {
		pop->locs = NULL ;
		pop->size = 0 ;
	}
	else {
		pop->locs = malloc(size*sizeof(struct S_RIGEL_Location)) ;
		if(pop->locs==NULL) {
			free(pop) ;
			return NULL ;
		}
		pop->size = size ;
	}
	if(hist==0) {
		pop->ancs = NULL ;
		pop->hist = 0 ;
	}
	else {
		pop->ancs = malloc(hist*sizeof(struct S_RIGEL_Ancestor)) ;
		if(pop->ancs==NULL) {
			free(pop->locs) ;
			free(pop) ;
			return NULL ;
		}
		pop->hist = hist ;
	}
	return pop ;
}

void _RIGEL_PopulationFree(RIGEL_Population pop) {
	if(pop!=NULL) {
		RIGEL_ParametersFree(pop->param) ;
		pop->param = NULL ;
		free(pop->ancs) ;
		pop->ancs = NULL ;
		pop->hist = 0 ;
		free(pop->locs) ;
		pop->locs = NULL ;
		pop->size = 0 ;
		pop->date = -1 ;
		pop->nwid = -1 ;
	}
}


int _RIGEL_LocCmpAscend(const void * l1, const void * l2) {
	double f1, f2 ;
	f1 = ((struct S_RIGEL_Location *)l1)->eval ;
	f2 = ((struct S_RIGEL_Location *)l2)->eval ;
	if(f1<f2) return -1 ;
	if(f1>f2) return +1 ;
	return 0 ;
}

int _RIGEL_LocCmpDescend(const void * l1, const void * l2) {
	double f1, f2 ;
	f1 = ((struct S_RIGEL_Location *)l1)->eval ;
	f2 = ((struct S_RIGEL_Location *)l2)->eval ;
	if(f1<f2) return +1 ;
	if(f1>f2) return -1 ;
	return 0 ;
}

unsigned long _RIGEL_PopSelProp(struct S_RIGEL_Location * locs, unsigned long leng, double step) {
	unsigned long i ;
	double s = 0 ;
	double t = 0 ;
	i = 0 ;
	while(i<leng) { s += locs[i].eval ; i ++ ; }
	s += step*leng ;
	s *= ((double)rand())/RAND_MAX ;
	i = 0 ;
	while(i<leng) { t += locs[i].eval+step ; if(t>s) return i ; i ++ ; }
	return i ;
}

unsigned long _RIGEL_PopSelRank(struct S_RIGEL_Location * locs, unsigned long leng, double step) {
	unsigned long i ;
	double s = 0 ;
	double t = 0 ;
	s = ((0.5+step)*leng*(leng+1)) * ((double)rand())/RAND_MAX ;
	i = 0 ;
	while(i<leng) { t += i+step ; if(t>s) return i ; i ++ ; }
	return i ;
}

unsigned long _RIGEL_PopSelTour(struct S_RIGEL_Location * locs, unsigned long leng, double numb) {
	unsigned long n ;
	unsigned long p, i ;
	unsigned long * t ;
	double e ;
	n = ceil(numb) ;
	// Tournament with less than two members doesn't exist
	if(n<2) n = 2 ;
	t = alloca(n*sizeof(unsigned long)) ;
	if(t==NULL) return 0 ;
	p = 0 ;
	while(p<n) { t[p] = floor((((double)rand())*leng)/RAND_MAX) ; p ++ ; }
	p = 0 ;
	e = locs[t[0]].eval ;
	while(p<n) { if(e<locs[t[p]].eval) e = locs[t[p]].eval ; p ++ ; }
	p = 0 ;
	i = 0 ;
	while(p<n) { if(e==locs[t[p]].eval) { t[i] = t[p] ; i ++ ; } p ++ ; }
	return t[0] ;
}





//################################################################################
//###
//###  Population dynamics methods
//###


RIGEL_Population RIGEL_Initialize(RIGEL_Interface iface, RIGEL_Parameters param) {
	RIGEL_Population pop ;
	unsigned long i ;
	if(param->popu_size==0) {
		errno = EINVAL ;
		DbgLog("RIGEL_Initialize","RIGEL_arg",0,"Invalid population size") ;
		return NULL ;
	}
	pop = _RIGEL_PopulationAlloc(param->popu_size,0) ;
	if(pop==NULL) {
		DbgLog("RIGEL_Initialize","RIGEL_mem",0,NULL) ;
		return NULL ;
	}
	pop->iface = iface ;
	pop->param = param ;
	pop->date = 1 ;
	pop->nwid = 1 ;
	i = 0 ;
	while(i<pop->size) {
		pop->locs[i].ident = pop->nwid ++ ;
		pop->locs[i].indiv = iface.Initialize(param) ;
		pop->locs[i].repr = 0 ;
		pop->locs[i].eval = pop->iface.Evaluate(pop->locs[i].indiv) ;
//		pop->locs[i].eval = NAN ;
		pop->locs[i].zone = 0.0 ;
		pop->locs[i].rate = 0.0 ;
		pop->locs[i].conf = 0.0 ;
		i ++ ;
	}
	return pop ;
}


RIGEL_Population RIGEL_Evaluate(RIGEL_Population pop) {
	unsigned long i ;
	if(pop==NULL) {
		errno = EINVAL ;
		 DbgLog("RIGEL_Evaluate","RIGEL_arg",0,"NULL population") ;
		 return NULL ;
	}
	i = 0 ;
	while(i<pop->size) {
		pop->locs[i].eval = pop->iface.Evaluate(pop->locs[i].indiv) ;
		i ++ ;
	}
	return pop ;
}


RIGEL_Population RIGEL_Generation(RIGEL_Population pop) {
	unsigned long size ;			// Size of intermediary population
	unsigned long beg, end ;
	struct S_RIGEL_Location * locs ;	// Intermediary population
	struct S_RIGEL_Ancestor * ancs ;	// Intermediary history
	unsigned long nwid = 0 ;		// New free ID
	RIGEL_Parameters nwpar ;		// New parameters
	RIGEL_Population nwpop ;		// New population
	unsigned long i, j ;
	if(pop==NULL) {
		errno = EINVAL ;
		DbgLog("RIGEL_Population","RIGEL_arg",0,"NULL population") ;
		return NULL ;
	}
	if(pop->size==0) {
		errno = EINVAL ;
		DbgLog("RIGEL_Population","RIGEL_arg",0,"Invalid population size") ;
		return NULL ;
	}
	if(pop->iface.Evaluate==NULL) {
		errno = EINVAL ;
		DbgLog("RIGEL_Population","RIGEL_arg",0,"Invalid population evaluator") ;
		return NULL ;
	}

	// Evaluating population
	i = 0 ;
	while(i<pop->size) {
		if(pop->locs[i].eval==NAN || (pop->param->popu_flag)&RIGEL_PARAM_EVALEACH) {
			pop->locs[i].eval = pop->iface.Evaluate(pop->locs[i].indiv) ;
		}
		i ++ ;
	}
	
	// Sorting population
	if((pop->param->popu_flag)&RIGEL_PARAM_SORTDESC)
		qsort(pop->locs,pop->size,sizeof(struct S_RIGEL_Location),_RIGEL_LocCmpDescend) ;
	else 
		qsort(pop->locs,pop->size,sizeof(struct S_RIGEL_Location),_RIGEL_LocCmpAscend) ;


	////////////////////////////////////////
	// Allocating intermediary population
	
	if(pop->nwid!=-1) nwid = pop->nwid ;
	size = pop->param->popu_elit+pop->param->popu_chld+pop->param->popu_init ;
	locs = malloc(size*sizeof(struct S_RIGEL_Location)) ;
	if(locs==NULL) {
		DbgLog("RIGEL_Population","RIGEL_mem",0,NULL) ;
		return NULL ;
	}
	ancs = malloc(size*sizeof(struct S_RIGEL_Ancestor)) ;
	if(ancs==NULL) {
		free(locs) ;
		DbgLog("RIGEL_Population","RIGEL_mem",0,NULL) ;
		return NULL ;
	}

	i = 0 ;	// Intermediary individual index


	////////////////////////////////////////
	// Writing elite in intermediary population

	while(i<pop->param->popu_elit) {
		// Keep the old ID if valid
		locs[i].ident = pop->locs[i].ident ;
		if(locs[i].ident==-1) locs[i].ident = nwid ++ ;
		// Copy indiv into loc
		if(pop->iface.Duplicate!=NULL) {
			locs[i].indiv = pop->iface.Duplicate(pop->locs[i].indiv) ;
		}
		else {
			locs[i].indiv = pop->locs[i].indiv ;
		}
		// Copy all data
		locs[i].repr = pop->locs[i].repr ;
		locs[i].eval = pop->locs[i].eval ;
		locs[i].zone = pop->locs[i].zone ;
		locs[i].rate = pop->locs[i].rate ;
		locs[i].conf = pop->locs[i].conf ;
		// Fill history
		ancs[i].ident = -1 ;
		// Search for previous record
		j = 0 ;
		while(j<pop->hist) {
			if(pop->ancs[j].ident == pop->locs[i].ident) {
				ancs[i] = pop->ancs[j] ;
				break ;
			}
			j ++ ;
		}
		// Record not found or invalid: recreate
		if(ancs[i].ident == -1) {
			ancs[i].ident = locs[i].ident ;
			ancs[i].parn1 = -1 ;
			ancs[i].parn2 = -1 ;
		}
		// Next loc
		i ++ ;
	}

	
	////////////////////////////////////////
	// Writing children in intermediary population

	// Compute begining of parents selection zone
	beg = 0 ;
	// In comma strategy, we don't use elite
	if(pop->param->popu_flag&RIGEL_PARAM_SELCOMMA) {
		beg = pop->param->popu_elit ;
		if(beg>=pop->size) beg = pop->size-1 ;
	}
	// Compute number of selected parents
	end = pop->param->popu_parn ;
	if((beg+end)>pop->size) end = pop->size-beg ;

	j = 0 ;	// Selection index for deterministic selection

	// Select parents, and compute children
	while(i<pop->param->popu_elit+pop->param->popu_chld) {
		unsigned long i1, i2 ;
		RIGEL_Genotype i_a1, i_a2, i_b1, i_b2, i_c1, i_c2, i_d, i_e, i_f ;
		locs[i].ident = nwid ++ ;
		// Select two parents
		switch(pop->param->popu_flag&RIGEL_PARAM_SELECTOR) {
			case RIGEL_PARAM_SELPROPN:
				// Proportionnal selection
				i1 = _RIGEL_PopSelProp(pop->locs+beg,end,pop->param->popu_fadj) ;
				i2 = _RIGEL_PopSelProp(pop->locs+beg,end,pop->param->popu_fadj) ;
				break ;
			case RIGEL_PARAM_SELRANKG:
				// Ranking selection
				i1 = _RIGEL_PopSelRank(pop->locs+beg,end,pop->param->popu_fadj) ;
				i2 = _RIGEL_PopSelRank(pop->locs+beg,end,pop->param->popu_fadj) ;
				break ;
			case RIGEL_PARAM_SELTOURN:
				// Tournament selection
				i1 = _RIGEL_PopSelTour(pop->locs+beg,end,pop->param->popu_fadj) ;
				i2 = _RIGEL_PopSelTour(pop->locs+beg,end,pop->param->popu_fadj) ;
				break ;
			default:
				// Deterministic selection
				i1 = j ++ ;
				if(j>=end) j = 0 ;	// Wrap around
				i2 = j ++ ;
				if(j>=end) j = 0 ;	// Wrap around
		}

		// Mutate
		if(pop->iface.Duplicate!=NULL && pop->iface.Paradigm&RIGEL_PARADIGM_MUTALLOC) {
			i_a1 = pop->iface.Duplicate(pop->locs[beg+i1].indiv) ;
			i_a2 = pop->iface.Duplicate(pop->locs[beg+i2].indiv) ;
		}
		else {
			i_a1 = pop->locs[beg+i1].indiv ;
			i_a2 = pop->locs[beg+i2].indiv ;
		}
		if(pop->iface.Mutate!=NULL) {
			i_b1 = pop->iface.Mutate(i_a1,pop->param) ;
			i_b2 = pop->iface.Mutate(i_a2,pop->param) ;
		}
		else {
			i_b1 = i_a1 ;
			i_b2 = i_a2 ;
		}

		// Crossover
		if(pop->iface.Duplicate!=NULL && pop->iface.Paradigm&RIGEL_PARADIGM_XOVALLOC) {
			i_c1 = pop->iface.Duplicate(i_b1) ;
			i_c2 = pop->iface.Duplicate(i_b2) ;
		}
		else {
			i_c1 = i_b1 ;
			i_c2 = i_b2 ;
		}
		if(pop->iface.CrossOver!=NULL) {
			i_d = pop->iface.CrossOver(i_c1,i_c2,pop->param) ;
		}
		else i_d = i_c1 ;

		// Migrate
		if(pop->iface.Duplicate!=NULL && pop->iface.Paradigm&RIGEL_PARADIGM_MIGALLOC) {
			i_e = pop->iface.Duplicate(i_d) ;
		}
		else i_e = i_d ;
		if(pop->iface.Migrate!=NULL) {
			i_f = pop->iface.Migrate(i_e,pop->param) ;
		}
		else i_f = i_e ;

		// Free intermediary steps
		if(pop->iface.Duplicate!=NULL && pop->iface.Destroy!=NULL
		   && !(pop->iface.Paradigm&RIGEL_PARADIGM_NODELETE)) {
			if(i_a1!=pop->locs[beg+i1].indiv && i_a1!=i_f) pop->iface.Destroy(i_a1) ;
			if(i_a2!=pop->locs[beg+i2].indiv && i_a2!=i_f) pop->iface.Destroy(i_a2) ;
			if(i_b1!=i_a1 && i_b1!=i_f) pop->iface.Destroy(i_b1) ;
			if(i_b2!=i_a2 && i_b2!=i_f) pop->iface.Destroy(i_b2) ;
			if(i_c1!=i_b1 && i_c1!=i_f) pop->iface.Destroy(i_c1) ;
			if(i_c2!=i_b2 && i_c2!=i_f) pop->iface.Destroy(i_c2) ;
			if(i_d!=i_c1 && i_d!=i_c2 && i_d!=i_f) pop->iface.Destroy(i_d) ;
			if(i_e!=i_d && i_e!=i_f) pop->iface.Destroy(i_e) ;
		}

		// Store result
		locs[i].indiv = i_f ;
		locs[i].repr = 0 ;
		locs[i].eval = pop->iface.Evaluate(locs[i].indiv) ;
		locs[i].zone = 0.0 ;
		locs[i].rate = 0.0 ;
		locs[i].conf = 0.0 ;

		// Fill history
		ancs[i].ident = locs[i].ident ;
		ancs[i].parn1 = pop->locs[beg+i1].ident ;
		ancs[i].parn2 = pop->locs[beg+i2].ident ;

		// Next loc
		i ++ ;
	}
	
	
	////////////////////////////////////////
	// Writing recreated individuals in intermediary population

	while(i<size) {

		// Initialize components
		locs[i].ident = nwid ++ ;
		locs[i].indiv = pop->iface.Initialize(pop->param) ;
		locs[i].repr = 0 ;
		locs[i].eval = pop->iface.Evaluate(locs[i].indiv) ;
		locs[i].zone = 0.0 ;
		locs[i].rate = 0.0 ;
		locs[i].conf = 0.0 ;

		// Fill history
		ancs[i].ident = locs[i].ident ;
		ancs[i].parn1 = -1 ;
		ancs[i].parn2 = -1 ;

		// Next loc
		i ++ ;
	}


	////////////////////////////////////////
	// Allocating new population

	// Allocating new parameters
	nwpar = RIGEL_ParametersCopy(pop->param) ;
	if(nwpar==NULL) { free(locs) ; free(ancs) ; return NULL ; }
	if(nwpar->popu_size==0) {
		nwpar->popu_size = pop->size ;
	}
	// Computing new population size and history
	i = pop->hist+pop->param->popu_chld ;
	if(i>nwpar->popu_hist) i = nwpar->popu_hist ;
	// Allocating new population
	nwpop = _RIGEL_PopulationAlloc(nwpar->popu_size,i) ;
	if(nwpop==NULL) { free(locs) ; free(ancs) ; return NULL ; }
	nwpop->iface = pop->iface ;
	nwpop->param = nwpar ;
	nwpop->date = pop->date + 1 ;
	nwpop->nwid = nwid ;

	i = 0 ;	// Final individual index

	////////////////////////////////////////
	// Writing final population

	if(pop->param->popu_flag&RIGEL_PARAM_REPCOMMA) {
		// Comma strategy: elite is kept
		while(i<pop->param->popu_elit) {
			// Copy all data
			nwpop->locs[i] = locs[i] ;
			// Mark old indiv as taken
			locs[i].ident = -1 ;
			// Next loc
			i ++ ;
		}
		// Sorting remaining population
		beg = pop->param->popu_elit ;
		end = size-beg ;
		if((pop->param->popu_flag)&RIGEL_PARAM_SORTDESC)
			qsort(locs+beg,end,sizeof(struct S_RIGEL_Location),_RIGEL_LocCmpDescend) ;
		else 
			qsort(locs+beg,end,sizeof(struct S_RIGEL_Location),_RIGEL_LocCmpAscend) ;
	}
	else {
		// Sorting population
		beg = 0 ;
		end = size ;
		if((pop->param->popu_flag)&RIGEL_PARAM_SORTDESC)
			qsort(locs,end,sizeof(struct S_RIGEL_Location),_RIGEL_LocCmpDescend) ;
		else
			qsort(locs,end,sizeof(struct S_RIGEL_Location),_RIGEL_LocCmpAscend) ;
	}

	j = 0 ;	// Selection index for deterministic selection

	// Filling population
	while(i<nwpop->size) {
		unsigned long i1 ;
		// Select offspring
		switch(pop->param->popu_flag&RIGEL_PARAM_REPLACMT) {
			case RIGEL_PARAM_REPPROPN:
				// Proportionnal selection
				i1 = _RIGEL_PopSelProp(locs+beg,end,pop->param->popu_fadj) ;
				break ;
			case RIGEL_PARAM_REPRANKG:
				// Ranking selection
				i1 = _RIGEL_PopSelRank(locs+beg,end,pop->param->popu_fadj) ;
				break ;
			case RIGEL_PARAM_REPTOURN:
				// Tournament selection
				i1 = _RIGEL_PopSelTour(locs+beg,end,pop->param->popu_fadj) ;
				break ;
			default:
				// Deterministic selection
				i1 = j ++ ;
				if(j>=end) j = 0 ;	// Wrap around
		}
		// Find first available individual
		if(locs[beg+i1].ident==-1) {
			// Search for available indiv
			while(i1<end && locs[beg+i1].ident==-1) i1 ++ ;
			if(i1>=end) while(i1>0 && locs[beg+i1-1].ident==-1) i1 -- ;
			// Not available -> abort filling
			if(locs[beg+i1].ident==-1) break ;
		}
		// Copy all data
		nwpop->locs[i] = locs[beg+i1] ;
		// Mark old indiv as taken
		locs[beg+i1].ident = -1 ;
		// Next loc
		i ++ ;
	}

	// Complete population by random individuals
	while(i<nwpop->size) {

		// Initialize components
		nwpop->locs[i].ident = nwpop->nwid ++ ;
		nwpop->locs[i].indiv = pop->iface.Initialize(nwpar) ;
		nwpop->locs[i].repr = 0 ;
		nwpop->locs[i].eval = pop->iface.Evaluate(nwpop->locs[i].indiv) ;
		nwpop->locs[i].zone = 0.0 ;
		nwpop->locs[i].rate = 0.0 ;
		nwpop->locs[i].conf = 0.0 ;

		// Next loc
		i ++ ;
	}

	////////////////////////////////////////
	// Fill history from this generation
	i = 0 ;
	j = 0 ;
	while(i<nwpop->size && j<nwpop->hist) {
		unsigned long k = 0 ;
		nwpop->ancs[j].ident = -1 ;
		// Search for previous record in this generation's history
		k = 0 ;
		while(k<size) {
			if(nwpop->locs[i].ident == ancs[k].ident) {
				nwpop->ancs[j] = ancs[k] ;
				break ;
			}
			k ++ ;
		}
		// Valid record: OK
		if(ancs[j].ident != -1) j ++ ;
		// Next loc
		i ++ ;
	}

	////////////////////////////////////////
	// Finish filling up history
	if(pop->hist>pop->param->popu_elit) {
		i = 0 ;
		while(i<j && j<nwpop->hist) {
			unsigned long k ;
			// Find parent 1 history
			k = 0 ;
			while(k<pop->hist) {
				if(nwpop->ancs[i].parn1 == pop->ancs[k].ident) break ;
				k ++ ;
			}
			if(k<pop->hist) {
				nwpop->ancs[j] = pop->ancs[k] ;
				// Check if it isn't there already
				k = 0 ;
				while(k<j) {
					if(nwpop->ancs[j].ident == nwpop->ancs[k].ident) break ;
				}
				// If not found, keep it
				if(k==j) { j ++  ; }
			}
			if(j==nwpop->hist) break ;
			// Find parent 2 history
			k = 0 ;
			while(k<pop->hist) {
				if(nwpop->ancs[i].parn2 == pop->ancs[k].ident) break ;
				k ++ ;
			}
			if(k<pop->hist) {
				nwpop->ancs[j] = pop->ancs[k] ;
				// Check if it isn't there already
				k = 0 ;
				while(k<j) {
					if(nwpop->ancs[j].ident == nwpop->ancs[k].ident) break ;
				}
				// If not found, keep it
				if(k==j) { j ++  ; }
			}

			// Next child
			i ++ ;
		}
	}
	
	// Force history limit if we did not fill archive
	if(j<nwpop->hist) nwpop->hist = j ;

	// Finish cleaning up
	if(pop->iface.Duplicate!=NULL && pop->iface.Destroy!=NULL
	   && !(pop->iface.Paradigm&RIGEL_PARADIGM_NODELETE)) {
		i = 0 ;
		while(i<size) {
			if(locs[i].ident!=-1) {
				j = 0 ;
				while(j<pop->size) {
					if(pop->locs[j].indiv==locs[i].indiv) break ;
					j ++ ;
				}
				if(j>=pop->size) pop->iface.Destroy(locs[i].indiv) ;
				i ++ ;
			}
			i ++ ;
		}
	}
	free(locs) ;
	free(ancs) ;

	return nwpop ;
}


void RIGEL_Terminate(RIGEL_Population pop) {
	unsigned long i = 0 ;
	if(pop == NULL) return ;
	while(i<pop->size) {
		if(pop->iface.Destroy!=NULL && !pop->iface.Paradigm&RIGEL_PARADIGM_NODELETE) {
			pop->iface.Destroy(pop->locs[i].indiv) ;
		}
		i ++ ;
	}
	_RIGEL_PopulationFree(pop) ;
}





//################################################################################
//###
//###  Population items accessors
//###



RIGEL_Parameters RIGEL_Getparameters(RIGEL_Population pop) {
	if(pop==NULL) {
		errno = EINVAL ;
		DbgLog("RIGEL_Getparameters","RIGEL_arg",0,"NULL population argument") ;
		return NULL ;
	}
	return pop->param ;
}


RIGEL_Population RIGEL_SetParameters(RIGEL_Population pop, RIGEL_Parameters param) {
	if(pop==NULL) {
		errno = EINVAL ;
		DbgLog("RIGEL_Setparameters","RIGEL_arg",0,"NULL population argument") ;
		return NULL ;
	}
	if(pop->param!=NULL) RIGEL_ParametersFree(pop->param) ;
	pop->param = param ;
	return pop ;
}


unsigned long RIGEL_GetIdentByRank(RIGEL_Population pop, unsigned long rank) {
	if(pop==NULL) {
		errno = EINVAL ;
		DbgLog("RIGEL_GetIdentByRank","RIGEL_arg",0,"NULL population argument") ;
		return -1 ;
	}
	if(rank>=pop->size) {
		errno = EINVAL ;
		DbgLog("RIGEL_GetIdentByRank","RIGEL_arg",0,"Invalid rank argument") ;
		return -1 ;
	}
	// Makes sure population is sorted
	if((pop->param->popu_flag)&RIGEL_PARAM_SORTDESC)
		qsort(pop->locs,pop->size,sizeof(struct S_RIGEL_Location),_RIGEL_LocCmpDescend) ;
	else 
		qsort(pop->locs,pop->size,sizeof(struct S_RIGEL_Location),_RIGEL_LocCmpAscend) ;
	return pop->locs[rank].ident ;
}


unsigned long RIGEL_GetIdentByEval(RIGEL_Population pop, double emin, double emax) {
	unsigned long i = 0 ;
	if(pop==NULL) {
		errno = EINVAL ;
		DbgLog("RIGEL_GetIdentByEval","RIGEL_arg",0,"NULL population argument") ;
		return -1 ;
	}
	while(i<pop->size) {
		if((pop->locs[i].eval>=emin)&&(pop->locs[i].eval<=emax)) return pop->locs[i].ident ;
		i ++ ;
	}
	return -1 ;
}


unsigned long RIGEL_GetGenealogy(RIGEL_Population pop, unsigned long ident, unsigned long * line, unsigned long gmax) {
	unsigned long g = 0 ; // Ancestors numbers < gmax
	unsigned long d = 0 ; // Depth of search =~ log2(gmax)
	unsigned long r ;     // father ident iterator
	unsigned long i ;     // search ident iterator
	if(pop==NULL) {
		errno = EINVAL ;
		DbgLog("RIGEL_GetGenealogy","RIGEL_arg",0,"NULL population argument") ;
		return -1 ;
	}
	if(gmax==0) return 0 ; 
	// First item (root) of the btree, i.e. row 0
	line[g] = ident ; g ++ ;
	while(g<(gmax-1)) {
		// Building row d+1 of the btree (with 2^(d+1) items)
		// iterating on ancestors from row d, i.e. between line+2^d-1 aond line+2^(d+1)-1
		r = 0 ;
		while(r<(1<<d)) {
			if(g>=(gmax-1)) return g ;
			// Producing a pair of items for each one in the row d
			i = 0 ;
			while(i<pop->hist) {
				// The ancestors are 
				if(pop->ancs[i].ident==line[(1<<d)-1+r]) {
					line[g] = pop->ancs[i].parn1 ; g ++ ;
					line[g] = pop->ancs[i].parn2 ; g ++ ;
					// We should have only one match
					break ;
				}
				i ++ ;
			}
			if(i==pop->hist) {
				// No match found -> fill with -1
				line[g] = -1 ; g ++ ;
				line[g] = -1 ; g ++ ;
				break ;
			}
			r ++ ;
		}
		d ++ ;
	}
	return g ;
}

RIGEL_Genotype RIGEL_GetItemByIdent(RIGEL_Population pop, unsigned long ident) {
	unsigned long i = 0 ;
	if(pop==NULL) {
		errno = EINVAL ;
		DbgLog("RIGEL_GetItemByIdent","RIGEL_arg",0,"NULL population argument") ;
		return NULL ;
	}
	while(i<pop->size) {
		if(pop->locs[i].ident == ident) return pop->locs[i].indiv ;
		i ++ ;
	}
	errno = EINVAL ;
	DbgLog("RIGEL_GetItemByIdent","RIGEL_arg",0,"Invalid ident argument") ;
	return NULL ;
}

RIGEL_Genotype RIGEL_GetItemByRank(RIGEL_Population pop, unsigned long rank) {
	if(pop==NULL) {
		errno = EINVAL ;
		DbgLog("RIGEL_GetItemByRank","RIGEL_arg",0,"NULL population argument") ;
		return NULL ;
	}
	if(rank>=pop->size) {
		errno = EINVAL ;
		DbgLog("RIGEL_GetItemByIdent","RIGEL_arg",0,"Invalid rank argument") ;
		return NULL ;
	}
	// Makes sure population is sorted
	if((pop->param->popu_flag)&RIGEL_PARAM_SORTDESC)
		qsort(pop->locs,pop->size,sizeof(struct S_RIGEL_Location),_RIGEL_LocCmpDescend) ;
	else 
		qsort(pop->locs,pop->size,sizeof(struct S_RIGEL_Location),_RIGEL_LocCmpAscend) ;
	return pop->locs[rank].indiv ;
}

